﻿using Falcon.SugarApi.Encryption;
using Falcon.SugarApi.JsonSerialize;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.DependencyInjection.Extensions;
using System;

namespace Falcon.SugarApi.ClaimTicket
{
    /// <summary>
    /// 服务扩展
    /// </summary>
    public static class ServiceCollectionExtend
    {
        /// <summary>
        /// 增加客户凭据支持
        /// </summary>
        /// <param name="services">服务集合</param>
        /// <param name="OptionBuilder">配置创建</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddClaimTicket(this IServiceCollection services,Action<ClaimTicketOptions>? OptionBuilder = null) {
            services.TryAddSingleton<IEncryption,AESProvider>();
            services.TryAddSingleton<IJsonSerialize,JsonSerialize.JsonSerialize>();
            services.TryAddSingleton<ITicketBuilder,TicketBuilder>();

            services.TryAddSingleton<ClaimTicketOptions>(p => {
                var option = new ClaimTicketOptions();
                OptionBuilder?.Invoke(option);
                return option;
            });
            services.AddControllers(op => {
                op.ModelBinderProviders.Insert(0,new ClaimTicketModelBindingProvider());
            });
            return services;
        }
    }
}
