﻿using System.Reflection;

namespace Falcon.SugarApi.DatabaseManager
{
    /// <summary>
    /// 数据库上下文字段加密支持
    /// </summary>
    public static class DbContextBaseEncryptExtend
    {
        /// <summary>
        /// 设置数据库上下文支持字段加密
        /// </summary>
        /// <param name="dbContext">数据库上下文</param>
        public static void SetEnableEncrypt(this DbContextBase dbContext) {
            dbContext.Aop.DataExecuting = (value,entity) => {
                var attr = entity.GetAttribute<EnableEncryptAttribute>();
                var enProvider = attr.EncryptType.Assembly.CreateInstance(attr.EncryptType.FullName) as IEncrypt;
                if(attr == null) {
                    return;
                }
                var newVal = enProvider.Encrypt(value.ToString());
                entity.SetValue(newVal);
            };
            dbContext.Aop.DataExecuted = (value,entity) => {
                foreach(var col in entity.EntityColumnInfos) {
                    var pName = col.PropertyName;
                    var attr = col.PropertyInfo.GetCustomAttribute<EnableEncryptAttribute>();
                    var enProvider = attr.EncryptType.Assembly.CreateInstance(attr.EncryptType.FullName) as IEncrypt;
                    if(attr == null) {
                        continue;
                    }
                    var newVal = enProvider.Decrypt(entity.GetValue(pName).ToString());
                    entity.SetValue(pName,newVal);
                }
            };

        }
    }
}
