﻿using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.DependencyInjection.Extensions;
using System;
using System.Collections.Generic;
using System.Linq;

namespace Falcon.SugarApi.ServiceCollectionExtend
{
    /// <summary>
    /// 单实例多接口注册
    /// </summary>
    public static class AddSingletonWithExtend
    {
        /// <summary>
        /// 单实例+多接口注册。
        /// </summary>
        /// <typeparam name="TInstance">实例的类型</typeparam>
        /// <param name="services">服务集合</param>
        /// <param name="serviceTypes">要注册的服务类型,必须为接口</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddSingletonWith<TInstance>(this IServiceCollection services,params Type[] serviceTypes)
            where TInstance : class {
            services.TryAddSingleton<TInstance>();
            foreach(var t in serviceTypes) {
                if(!t.IsInterface)
                    throw new ArgumentException($"类型 {t.FullName} 不是接口");
                if(typeof(TInstance).GetInterfaces().Contains(t)) {
                    services.AddSingleton(t,p => p.GetRequiredService<TInstance>());
                }
            }
            return services;
        }

        /// <summary>
        /// 单实例+多接口注册。
        /// </summary>
        /// <typeparam name="TInstance">实例的类型</typeparam>
        /// <param name="services">服务集合</param>
        /// <param name="instance">实例对象</param>
        /// <param name="serviceTypes">要注册的接口类型</param>
        /// <returns>服务集合</returns>
        public static IServiceCollection AddSingletonWith<TInstance>(this IServiceCollection services,TInstance instance,params Type[] serviceTypes)
            where TInstance : class {
            services.TryAddSingleton<TInstance>(instance);
            foreach(var t in serviceTypes) {
                if(typeof(TInstance).GetInterfaces().Contains(t)) {
                    services.AddSingleton(t,p => p.GetRequiredService<TInstance>());
                }
            }
            return services;
        }
    }
}
